#pragma rtGlobals=1		// Use modern global access method.
#pragma version=1.41

// written by tony withers, for those who like to use totally arbitrary baselines.
// seems like a bad idea to me to use something so subjective. good luck!

// requires Igor Pro 6.0.1


// 1.41 10/9/07
// for XY data we now interpolate directly into XY baseline
// improved guesses for initial node positions for unevenly spaced XY data

// 1.40   10/9/07
// fixed bug that occurred with baseline subtraction of XY data

// 1.30  7/24/07
// handles XY data
// baseline is waveform, but data wave need not be (though it should be reasonably closely spaced).

// 1.20 7/3/07
// uses new graphwaveedit flag. now requires version 6.0.1.

Menu "Macros"
	Submenu "Spline Baseline"
		"Initalise...", ACW_splineBaseline()		
		ACW_ToggleFitMenu(), /Q, ACW_ToggleFitting()
		ACW_SubMenu(), ACW_subtractSpline()
	end
end

function /S ACW_ToggleFitMenu()	
	if (exists("root:Packages:SplineFit:V_edit")==2)
		NVAR V_edit= root:Packages:SplineFit:V_edit
		if (V_edit)
			return "Adjust Nodes!"+num2char(18)+"/1"
		else
			return "Adjust Nodes/1"
		endif	
	else
		return ""
	endif
end 


function /S ACW_SubMenu()
	if (exists("root:Packages:SplineFit:S_Data")==2)
		return "Subtract base..."
	else
		return ""
	endif
end


function ACW_splineBaseline()
	
	NewDataFolder /O root:Packages
	NewDataFolder /O root:Packages:SplineFit
	
	string  s_DataWave
	prompt s_datawave, "Data Wave", popup, WaveList("*",";","WIN:") 
	DoPrompt "", s_datawave
	if (V_Flag)
		return 0
	endif
	if (stringmatch(s_datawave, "_none_"))
		doalert 0, "No spectra in top graph window. Inititalisation failed."
		return 0
	endif
	
	string /G root:Packages:SplineFit:S_Data
	SVAR S_Data=root:Packages:SplineFit:S_Data
	S_Data=getdatafolder(1)+s_datawave
	
	wave data=$S_Data
	
	string /G root:Packages:SplineFit:S_xxx
	SVAR S_xxx=root:Packages:SplineFit:S_xxx
	
	// create a wave to store some global variables	
	if (exists("root:Packages:SplineFit:SplineFitGlobals")==1)
		wave SplineFitGlobals=root:Packages:SplineFit:SplineFitGlobals
		// for backward compatibility with older versions of this procedure
		redimension /N=6 SplineFitGlobals
		setdimlabel  0, 5, isXY, SplineFitGlobals
	else
		make /o/n=6 root:Packages:SplineFit:SplineFitGlobals=nan
		wave SplineFitGlobals=root:Packages:SplineFit:SplineFitGlobals
		setdimlabel  0, 0, null, SplineFitGlobals
		setdimlabel  0, 1, flagE, SplineFitGlobals
		setdimlabel  0, 2, flagF, SplineFitGlobals
		setdimlabel  0, 3, flagJ, SplineFitGlobals
		setdimlabel  0, 4, flagT, SplineFitGlobals
		setdimlabel  0, 5, isXY, SplineFitGlobals
		SplineFitGlobals[%flagE]=2
		SplineFitGlobals[%flagF]=1
		SplineFitGlobals[%flagJ]=1
		SplineFitGlobals[%flagT]=2
	endif
	
	SplineFitGlobals[%isXY]=0
	// find out whether data is XY
	S_xxx=StringByKey("XWAVE", TraceInfo("",s_datawave,0))
	if (strlen(S_xxx))
		wave xxx=XWaveRefFromTrace("", s_datawave )
		S_xxx[0]=StringByKey("XWAVEDF", TraceInfo("",s_datawave,0))
		SplineFitGlobals[%isXY]=1
	endif
		
	make /O/n=11 root:Packages:SplineFit:W_nodesX, root:Packages:SplineFit:W_nodesY
	make /O/n=1  root:Packages:SplineFit:W_spline_dependency
	wave W_nodesX=root:Packages:SplineFit:W_nodesX
	wave W_nodesY=root:Packages:SplineFit:W_nodesY
	wave W_spline_dependency=root:Packages:SplineFit:W_spline_dependency
	
	// initialise note positions. set 11 nodes to roughly follow the data
	variable tenthRange
	if (SplineFitGlobals[%isXY])
		tenthRange=(xxx[numpnts(xxx)-1]-xxx[0])/10
		W_nodesX=xxx[0]+p*tenthRange
	else
		tenthRange=(pnt2x(data, numpnts(data)-1)-pnt2x(data, 0))/10
		W_nodesX=pnt2x(data, 0)+p*tenthRange
	endif
	
	variable i
	variable pmin, pmax
	for (i=0;i<11;i+=1)
		if (SplineFitGlobals[%isXY])
			findlevel /Q/P xxx, W_nodesX[i]-tenthRange/6
			if (V_flag)
				pmin=0
			else
				pmin= V_LevelX
			endif
			
			findlevel /Q/P xxx, W_nodesX[i]+tenthRange/6
			if (V_flag)
				pmax=numpnts(xxx)
			else
				pmax= V_LevelX
			endif		
		else		
			pmin=  max(0, numpnts(data)/10*i-numpnts(data)/50) 
			pmax= min(numpnts(data), numpnts(data)/10*i+numpnts(data)/50 )
		endif
		wavestats /Q/R=[pmin , pmax] /Z  data
		W_nodesY[i]=V_avg  // this handles a few NANs	
	endfor
	
	// set a dependency to trigger interpolation AND update graph when we adjust a node position
	W_spline_dependency:=ACW_RefitSpline(root:Packages:SplineFit:W_nodesY)
	
	duplicate /O data root:Packages:SplineFit:W_base
	wave W_base=root:Packages:SplineFit:W_base
	
	removefromgraph /Z W_nodesY, w_base, W_spline_dependency
	
	AppendToGraph W_spline_dependency // need to have this on graph to force update while in drawing mode
	ModifyGraph hideTrace(W_spline_dependency)=1
	
	appendtograph W_nodesY vs W_nodesX
	ModifyGraph mode(W_nodesY)=3,marker(W_nodesY)=11,rgb(W_nodesY)=(0,39168,0)
	
	ACW_RefitSpline(W_nodesX)
	
	
	if (SplineFitGlobals[%isXY])
		appendtograph W_base vs xxx
	else
		appendtograph W_base
	endif
	
	ModifyGraph rgb(W_base)=(0,0,52224)
	modifygraph live(W_base)=1

	ReorderTraces $"W_nodesY",{$"W_base"}
	GraphWaveEdit /T=1 /M W_nodesY
	variable /G root:Packages:SplineFit:V_edit=1
	
	BuildMenu "macros"	
end

function ACW_RefitSpline(w)
	wave w
	
	wave g=root:Packages:SplineFit:SplineFitGlobals
	SVAR S_xxx=root:Packages:SplineFit:S_xxx

	// interpolate2 doesn't work unless you give full paths	
	if (g[%isXY])	
		Interpolate2 /E=(g[%flagE]) /F=(g[%flagF]) /J=(g[%flagJ]) /T=(g[%flagT]) /I=3  /X=$S_xxx /Y=root:Packages:SplineFit:W_base root:Packages:SplineFit:W_nodesX, root:Packages:SplineFit:W_nodesY
	else
		Interpolate2 /E=(g[%flagE]) /F=(g[%flagF]) /J=(g[%flagJ]) /T=(g[%flagT]) /I=3  /Y=root:Packages:SplineFit:W_base root:Packages:SplineFit:W_nodesX, root:Packages:SplineFit:W_nodesY
	endif
	
	return nan	
end


function ACW_subtractSpline()
	
	SVAR s_data=root:Packages:SplineFit:S_Data
	wave W_base=root:Packages:SplineFit:W_base
	wave data=$S_Data
	string strSubtracted, strBaseline
	strSubtracted=CleanupName( nameofwave(data)+"_BaselineSubtracted",0)
	strBaseline=CleanupName( nameofwave(data)+"_Baseline",0)

	if (exists(strSubtracted))
		doalert 1, strSubtracted+" exists. Overwrite?"
		if(V_flag==2)
			return 0
		endif
	endif
	duplicate /o data $strSubtracted
	
	if (exists(strBaseline))
		doalert 1, strBaseline+" exists. Overwrite?"
		if(V_flag==2)
			return 0
		endif
	endif
	duplicate /o W_base $strBaseline
	
	wave nob=$strSubtracted
	wave bl=$strBaseline
	
	nob=data-bl
	
	removefromgraph /Z $strSubtracted, $strBaseline, W_base,  W_nodesY
	
	// find out whether data is XY	
	string Xwave=StringByKey("XWAVE", TraceInfo("",NameOfWave(data),0))
	
	if (strlen(Xwave))  // XY data
		wave xxx=XWaveRefFromTrace("", NameOfWave(data) )	
		appendtograph nob, bl vs xxx
	else
		appendtograph nob, bl
	endif
	
	ModifyGraph rgb($nameofwave(bl))=(0,65280,0)	
	ModifyGraph rgb($nameofwave(nob))=(0,0,0)	
	killvariables /z root:Packages:SplineFit:V_edit
	killstrings /z root:Packages:SplineFit:S_Data
	BuildMenu "macros"	

end

function ACW_ToggleFitting()

	NVAR V_edit=root:Packages:SplineFit:V_edit
	if (V_edit)
		GraphNormal
		ModifyGraph mode(W_nodesY)=2
		V_edit=0
	else
		ModifyGraph mode(W_nodesY)=3
		GraphWaveEdit /T=1 /M $"W_nodesY"
		V_edit=1
	endif
	buildmenu "macros"
end